# This code is copyrighted by bluedynamics.com and iuveno AG in May, 2001.
#
# written by Stephan Richter
# Last updated: May 19, 2001

# This class really does not do all of the copy mechanism, since the ObjectManager via the CopyContainer
# supports most of the functionality. In this class, I will just restrict these functions a little, so
# that you cannot copy a User into a Project, for example. Also, the DB containerid and containertype
# will be adjusted to the new location.

from OFS import Moniker
from OFS.CopySupport import CopyContainer, CopyError, _cb_decode, eNoData, eInvalid, eNotFound, eNotSupported


class DBObjectCopySupport:
    '''DBObjectCopySupport class - Support to allow DBObejcts to be copied correctly'''

    manage_pasteObjectFingerPrint = CopyContainer.manage_pasteObjects

    # The reason I am not changing the parameters is to stay Zope API compatible!
    def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None, returnId=0):
        '''manage_pasteObjects(self, cb_copy_data=None, REQUEST=None) --> Paste previously copied objects
           into the current object; return None or HTML;'''

        # Alright, first we try to get the copy info in text format from the passed variable or the
        # REQUEST (by Cookie)
        # Note: A lot of the following code is copied directly from the CopyContainer stuff. I changed
        # some of the variable names, so it is more descriptive and follows DBObject's style guide.
        copyInfo = None
        if cb_copy_data is not None:
            copyInfo = cb_copy_data
        else:
            if REQUEST and REQUEST.has_key('__cp'):
                copyInfo = REQUEST['__cp']

        # No info found, so throw an error
        if copyInfo is None:
            raise CopyError, eNoData

        # Now try to make some sense out of the text based info; if not, throw an exception
        try:
            copyInfo = _cb_decode(copyInfo)
        except:
            raise CopyError, eInvalid

        objectList = []                           # Initalize object list
        operation  = copyInfo[0]                  # Get the operation: copy (0) or move (1)
        objectRoot = self.getPhysicalRoot()       # Get Object Root, so we have a reference

        # Wow, let's get the objects from the copy info; The Moniker code is really brief and cool;
        # you should check it out! My RDB2ObjectSupport is basically the same for DBObjects
        for monikerData in copyInfo[1]:
            moniker = Moniker.loadMoniker(monikerData)
            try:
                object = moniker.bind(objectRoot)
            except:
                raise CopyError, eNotFound
            self._verifyObjectPaste(object)
            objectList.append(object)

        if operation == 0:
            # Copy objects in the SQL table

            newIds = []
            html = ''
            
            for object in objectList:
                if object.meta_type in self.allowedSubObjects:
                    attributes  = object.getAttributes()[0]

                    if not REQUEST:
                        REQUEST = attributes
                    else:
                        for name in attributes.__record_schema__.keys():
                            REQUEST.set(name, attributes[name])
                        
                    if returnId:
                        newIds.append(self.manage_add(self, REQUEST, 1, object.meta_type))
                    else:
                        html = self.manage_add(REQUEST, 0, object.meta_type)

                else:
                    raise CopyError, eNotSupported % self.id

            if returnId:
                return newIds
            else:
                return html

        elif operation == 1:
            # Move objects in the SQL table
            for object in objectList:
                dbContainer = self.getDBContainerInfo()
                REQUEST.set('containerid', dbContainer[0])
                REQUEST.set('containertype', dbContainer[1])

                object.manage_edit(REQUEST, 0)

            return self.manage_pasteObjectFingerPrint(cb_copy_data, REQUEST)

        return None







