# This code is copyrighted by Stephan Richter in August, 2000.
#
# written by Stephan Richter
# Last updated: August 12, 2000

import string

class ZQuestException:
    '''PlanDepot Exception class - general PlanDepot Error class'''

    def __init__(self):
        '''__init__(self) --> setup common variables; return None'''

        self.message = 'PlanDepot Exception: '


    def __str__(self):
        '''__str__(self) --> return the string representation of the Exception.'''

        return self.message


    def __call__(self):
        '''__call__(self) --> return the string representation of the Exception.'''

        return self.message



class WrongMetaType(ZQuestException):
    '''WrongMetaType class - Error: Raised when an incorrect object was passed.'''

    def __init__(self, allowedMetaTypes, objectMetaType):
        '''__init__(self, allowedMetaTypes, objectMetaType) --> Initialize the object; return None'''
        
        # name of the meta-type that was expected
        self.allowedMetaTypes = allowedMetaTypes

        # name of the passed object's meta-type
        self.objectMetaType = objectMetaType

        # Error Message 
        self.message = ("Allowed Meta-Types: " + string.join(self.allowedMetaTypes, ', ') +
                        "; Object's Meta-Type: " + self.objectMetaType)



class MissingAttributes(ZQuestException):
    '''MissingAttributes class - Error: Raised, if required attributes for an object were not found.'''

    def __init__(self, objectName, attributeName):
        '''__init__(self, objectName, attributeName) --> Initialize the object; return None'''
        
        # the class name of the object that was attempted to be added/edited 
        self.objectName = objectName

        # the attributes name that was missing
        self.attributeName = str(attributeName)

        # Error Message 
        self.message = "Unable to manipulate object type '%s', since attribute '%s' is missing." %(
            self.objectName, attributeName)
        


class InitializationFailure(ZQuestException):
    """InitializationFailure class - Error: Raised, if an object's initialization (__init__) fails for
       some reason."""

    def __init__(self, objectMetaType):
        '''__init__(self, objectMetaType) --> Initialize the object; return None'''

        # The meta-type of the object which's initialization failed.
        self.objectMetaType = objectMetaType

        # Error Message 
        self.message = 'Initialization of ' + self.objectMetaType + ' failed.'



class IllegalSubObjectMetaType(ZQuestException):
    '''IllegalSubObjectMetaType class - Error: Raised if the requested meta-type was not found
       in the allowedSubObjectMetaTypes.'''

    def __init__(self, allowedSubObjectMetaTypes, requestedSubObjectMetaType):
        '''__init__(self, allowedSubObjectMetaTypes, requestedSubObjectMetaType) --> Initialize the
           object; return None'''

        # save the list of all allowed meta-types
        self.allowedSubObjectMetaTypes = allowedSubObjectMetaTypes

        # save the meta-type that was passed 
        self.requestedSubObjectMetaType = requestedSubObjectMetaType

        # Error Message
        self.message = ('Allowed Meta-Types: ' + string.join(self.allowedSubObjectMetaTypes, ', ') +
                        '; Requested Meta-Type: ' + self.requestedSubObjectMetaType)



class NoAllowedSubObjectMetaTypes(ZQuestException):
    '''NoAllowedSubObjectMetaTypes class - Error: Raised if there are no meta-types defined for the
       object.'''

    def __init__(self):
        '''__init__(self) --> Initialize the object; return None'''

        # Error Message
        self.message = 'There are no sub-object meta-types defined for this object.'
