# This code is copyrighted by iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: February 22, 2001

from Products.ZSQLMethods.SQL import SQL
from Globals                  import HTMLFile


class DBObjectSQLContainerSupport:
    '''DBObjectSQLContainerSupport class - adds some convinient functionality to a SQL Container'''

    isVolatile = 0
    isSQLMethodContainer = 1

    manage_options = ( {'label':'SQL Methods',         'action':'manage_sql'},
                       {'label':'Init Tables/Views',   'action':'manage_initTablesForm'}
                       )

    manage_sql__roles__   = ('Manager',)
    manage_sql            = HTMLFile('dtml/manage_sql', globals())
    manage_initTablesForm__roles__ = ('Manager',)
    manage_initTablesForm          = HTMLFile('dtml/manage_initTablesForm', globals())
    manage_initTables__roles__ = ('Manager',)
    manage_initTables          = HTMLFile('dtml/manage_initTables', globals())


    def getSQLMethods(self):
        '''getSQLMethods(self) --> return a list of SQL Methods defined in ProjectFolder; return
           list of objects'''
        sqlMethods = []
        type       = SQL.meta_type
        
        for ob in self.__dict__.values():
            if hasattr(ob, 'meta_type') and ob.meta_type==type:
                sqlMethods.append(ob)

        return sqlMethods


    def getInitSQLMethods(self):
        '''getInitSQLMethods(self) - get all the table initialization methods; return list '''

        initMethods = []

        for sqlMethod in self.getSQLMethods():
            if sqlMethod.id[:8] == 'sql_init':
                initMethods.append(sqlMethod)

        return initMethods


    def getInitSQLMethodsIds(self):
        '''getInitSQLMethodsIds(self) - get all the table initialization methods ids; return list '''

        return map(lambda o: o.id, self.getInitSQLMethods())


    def createAllSQLMethods(self, REQUEST=None, connectionId='dbConnection'):
        '''createAllSQLMethods(self, connectionId="dbConnection") --> create all SQL methods that are required
           for this system; return None or HTML'''
        self.initSQLMethods(self, connectionId)
        
        metaTypes = list(self.allowedSubObjects)
        for metaType in metaTypes:
            
            object = self.getSubObjectClass(metaType)

            if hasattr(object, 'allowedSubObjects'):
                for objectMetaType in object.allowedSubObjects:
                    if objectMetaType not in metaTypes:
                        metaTypes.append(objectMetaType)

            if hasattr(object, 'initSQLMethods'):
                instance = object(parent=self, id='temporary')
                instance.initSQLMethods(self, connectionId)

        if REQUEST:
            return self.manage_sql(self, REQUEST)


    def deleteSQLMethods(self, ids=[], REQUEST=None):
        '''deleteSQLMethods(self, ids=[], REQUEST=None) --> delete the specified SQL methods;
        return HTML'''
        for id in ids:
            if hasattr(self, id):
                delattr(self, id)
                
        if REQUEST:
            return self.manage_sql( self, REQUEST )
