from Products.ZSQLMethods.SQL import SQL
from DBTable                  import DBTable, DBAttribute
from Globals                  import HTMLFile
# from Version                  import Version

import string

MaxNumOfSubNumber = 255L



class DBObjectUndoSupport:
    '''DBObjectUndoSupport class - Implments the standard Undo functionality for DBObjects''' 

    useUndo = 0

    historyDBTable = DBTable('History', [],
                             {'id'           : DBAttribute('id',         'int8', 'int'),
                              'version'      : DBAttribute('version',    'int8', 'int'),
                              'outdatedon'   : DBAttribute('outdatedon', 'datetime'),
                              'isarchived'   : DBAttribute('isarchived', 'boolean')} )

    __ac_permissions__ = ( ('View management screens', ('manage_UndoForm',) ),
                           ('Edit DBObjects',          ('manage_undo',)     ), ) 


    manage_options = ({'label':'Undo',    'action':'manage_UndoForm'},)
    manage_UndoForm     = HTMLFile('dtml/dbObject_manageUndoForm', globals())
    manage_showRevision = HTMLFile('dtml/dbObject_manageShowRevision', globals())


    def manage_undo(self, historyIds=[], REQUEST=None):
        ''' '''
        if historyIds:
            historyIds.sort()
            dbRow = self.getRevision(historyid=historyIds[-1])
            self.manage_edit(dbRow, forward=0)
            self.deleteRevisions(historyIds[-1])

        if REQUEST:
            return self.manage_UndoForm(self, REQUEST)


    def manage_addRevision(self, version='', REQUEST=None):
        ''' '''
        self.addRevision('t', version)
        
        if REQUEST:
            return self.manage_UndoForm(self, REQUEST)


    def manage_archiveRevisions(self, historyIds=[], REQUEST=None):
        ''' '''
        for id in historyIds:
            revision = self.getRevision(id)
            if revision['isarchived'] == 'f':
                self.setArchivedFlag(id, 't')
            else:
                self.setArchivedFlag(id, 'f')

        if REQUEST:
            return self.manage_UndoForm(self, REQUEST)


    def manage_deleteRevisions(self, historyIds=[], REQUEST=None):
        ''' '''
        for id in historyIds:
                self.deleteRevision(id)
                
        if REQUEST:
            return self.manage_UndoForm(self, REQUEST)
        

    def addRevision(self, isarchived='f', version=''):
        '''addRevisionOfObject(self, isarchived='f', version='') --> add a revision of an object;
           return None'''
        dbContainer = self._getSQLMethodContainer(self)
        attrs = {}
        columns = self.dbTable.getAllAttributeNames() + ['id']

        if not version:
            revisions = self.qry_getRevisionsOfObject(table=self.dbTable.name, id=self.id)
            if len(revisions):
                version = self.incrementVersionNumber(long(revisions[0]['version']))
            else:
                version = self.encodeVersionNumber('0.0.1')
        else:
            version = self.encodeVersionNumber(version)

        for column in columns:
            attrs[column] = self.getTemporaryAttributes()[column]

        dbContainer.sql_addRevision(table=self.dbTable.name, attrs=attrs, isarchived=isarchived,
                                    version=str(version))

        
    def setArchivedFlag(self, historyid, isarchived='t'):
        '''setArchivedFlag(self, historyid, isarchived=\'t\') --> '''
        dbContainer = self._getSQLMethodContainer(self)
        dbContainer.sql_setArchivedFlag(table=self.dbTable.name, historyid=historyid, isarchived=isarchived)


    def deleteRevision(self, historyid):
        '''deleteRevision(self, historyid) --> delete revision, if not archived;
           return None'''
        dbContainer = self._getSQLMethodContainer(self)
        dbContainer.sql_deleteRevision(table=self.dbTable.name, historyid=historyid)


    def deleteRevisions(self, historyid):
        '''deleteRevisions(self, historyid) --> delete all revizions since a date, which are not archived;
           return None'''
        dbContainer = self._getSQLMethodContainer(self)
        dbContainer.sql_deleteRevisions(table=self.dbTable.name, historyid=historyid)


    def deleteRevisionsOlderThan(self, dt):
        '''deleteRevisionsOlderThan(self, dt) --> delete all revizions since a date, which are not archived;
           return None'''
        dbContainer = self._getSQLMethodContainer(self)
        dbContainer.sql_deleteRevisionsOderThan(table=self.dbTable.name, dt=dt)


    def getRevision(self, historyid):
        '''getRevision(self) --> '''
        dbContainer = self._getSQLMethodContainer(self)
        return self.qry_getRevision(table=self.dbTable.name, historyid=historyid)[0]


    def getRevisions(self):
        '''getRevisions(self) --> '''
        dbContainer = self._getSQLMethodContainer(self)
        dbRows = self.qry_getRevisionsOfObject(table=self.dbTable.name, id=self.id)
        return self.makeObjectsFromRows(dbRows, self.aq_parent)


    def incrementVersionNumber(self, number, increment=1, separator='.'):
        ''' '''
        numberType = type(number).__name__
        if numberType == 'long int':
            number = self.decodeVersionNumber(number)

        nums = string.split(number, separator)
        num = int(nums[-1])
        num = num + increment
        nums[-1] = str(num)
        number = string.join(nums, separator)

        if numberType == 'long int':
            number = self.encodeVersionNumber(number)
            
        return number


    def getVersionRoot(self):
        ''' '''
        object = Version(self.id, '0.0.0')
        object._p_jar = self._p_jar
        object = object.__of__(self)
        return object
    

    def encodeVersionNumber(self, number, separator='.'):
        ''' '''
        nums = string.split(number, separator)
        hash = 0L
        for pos in range(len(nums)):
            hash = hash + int(nums[pos])*MaxNumOfSubNumber**(7-pos)

        return hash


    def decodeVersionNumber(self, hash, separator='.'):
        ''' '''
        hash = long(hash)
        numbers = []
        for pos in range(7, -1, -1):
            if not numbers and hash/MaxNumOfSubNumber**pos:
                numbers.append(int(hash/MaxNumOfSubNumber**pos))
            elif numbers:
                numbers.append(int(hash/MaxNumOfSubNumber**pos))

            hash = hash - long(hash/MaxNumOfSubNumber**pos)*MaxNumOfSubNumber**pos

        numbers = filter(lambda x: x!=0, numbers)
        numbers = numbers + [0]*(3-len(numbers))
        numbers = map(str, numbers)
        return string.join(numbers, '.')

    

_sql_addRevision = '''
INSERT INTO <dtml-var table>History
  (
    <dtml-in "attrs.keys()">
      &dtml-sequence-item;,
    </dtml-in>
    version,
    outdatedon,
    isarchived
    )
  VALUES (
    <dtml-in "attrs.values()">
      <dtml-sqlvar sequence-item type="string">,
    </dtml-in>
    <dtml-sqlvar version   type="int">,
    abstime('now'),
    <dtml-sqlvar isarchived type="nb">    
  )
'''

_sql_setArchivedFlag = '''
UPDATE <dtml-var table>History
   SET isarchived = <dtml-sqlvar isarchived type="string">
   WHERE <dtml-sqltest historyid type="int">
'''

_sql_deleteRevision = '''
DELETE FROM <dtml-var table>History
  WHERE historyid = <dtml-sqlvar historyid type="int">
    AND isarchived = 'f'
'''

_sql_deleteRevisions = '''
DELETE FROM <dtml-var table>History
  WHERE historyid >= <dtml-sqlvar historyid type="int">
    AND isarchived = 'f'
'''

_sql_deleteRevisionsOlderThan = '''
DELETE FROM <dtml-var table>History
  WHERE outdatedby > <dtml-sqlvar dt type="string">
    AND isarchived = 'false'
'''

_qry_getRevision = '''
SELECT *
  FROM <dtml-var table>History
  WHERE <dtml-sqltest historyid type="int">
'''

_qry_getRevisionsOfObject = '''
SELECT *
  FROM <dtml-var table>History
  WHERE <dtml-sqltest id type="int">
  ORDER BY outdatedon DESC
'''


class DBObjectFolderUndoSupport:
    '''DBObjectFolderUndoSupport class - Implments the standard Undo functionality for DBObjectFolder''' 

    useUndo = 1

    sqlUndoMethods = ( ('sql_addRevision', 'table attrs isarchived="f" version="" ',  _sql_addRevision),
                       ('sql_setArchivedFlag',          'table historyid isarchived', _sql_setArchivedFlag),
                       ('sql_deleteRevision',           'table historyid', _sql_deleteRevision),
                       ('sql_deleteRevisions',          'table historyid', _sql_deleteRevisions),
                       ('sql_deleteRevisionsOlderThan', 'table dt',        _sql_deleteRevisionsOlderThan),
                       ('qry_getRevision',              'table historyid', _qry_getRevision),
                       ('qry_getRevisionsOfObject',     'table id',        _qry_getRevisionsOfObject),
                       ) 


    def initUndoSQLMethods(self, container, subObject, connectionId='dbConnection'):
        '''initSQLMethods(self, container) --> initializes additional SQML Methods; return None'''

        dbAttrs = {}
        for attr in subObject.dbTable.getAllAttributes() + subObject.historyDBTable.getAllAttributes():
            dbAttrs[attr.name] = attr

        table = DBTable(subObject.dbTable.name+'History', [], dbAttrs)

        if not hasattr(container, 'sql_init%s' %table.name):
            setattr(container, 'sql_init%s' %table.name, table.makeInitTableSQLMethod(historyTable=1))

        for sqlMethod in self.sqlUndoMethods:
            if not hasattr(container, sqlMethod[0]):
                setattr(container, sqlMethod[0], SQL(sqlMethod[0], '', connectionId, sqlMethod[1], sqlMethod[2]))


    def manage_pack(self, startDateTime, REQUEST=None):
        ''' '''

        for object in self.objectValues():
            dbContainer = self._getSQLMethodContainer(self)
            dbRows = object.deleteRevisionsOlderThan(startDateTime)
        
        if REQUEST:
            return self.manage_packForm(self, REQUEST)






