# This code is copyrighted by Stephan Richter in August, 2000.
# This code is copyrighted by iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: March 18, 2001

from Globals                  import HTMLFile
from DBObject                 import DBObject
from OFS.ObjectManager        import ObjectManager
from types                    import StringType
from ZPublisher.HTTPResponse  import HTTPResponse

import string, General


class FolderishDBObject(ObjectManager, DBObject):
    '''Folderish class - implements DB objects that can also be folders'''

    # This dictionary contains meta-type - objects pairs of all the allowed sub-objects
    allowedSubObjects = ()

    # set the permissions required for executing a certain management method
    __ac_permissions__ = ( (
        ('View management screens',     ('manage_addForm',)),
        ('Access contents information', ('objectIds', 'objectValues', 'objectItems',''), ('Anonymous', 'Manager')),
        ('Add DBObjects',               ('manage_add', 'findExistingSubObjects',)),
	# edit permission is "inherited"
        ('Delete DBObjects',            ('manage_delObjects',)),
        ('View DBObjects',              ('objectItems', 'objectValues', 'objectIds')), )
	+ DBObject.__ac_permissions__ )

    manage_options = (({'label':'Contents',        'action':'manage_main'},)
                      + DBObject.manage_options)

    manage_main = HTMLFile('dtml/folder_manageContents', globals())
    
    
    def __bobo_traverse__(self, REQUEST, name):
        '''__bobo_traverse__(self, REQUEST, name) --> traverse through the object; return object or HTML'''

        # first see, whether the object with the id name exists in here
        try:
            return self._getObjectSafely(name)
        except 'NotFound':
            pass

        # if it is not an object, try whether it is an attribute
        try:
            return getattr(self, name)
        except AttributeError:
            pass

        #for object in self.objectValues():
        #    try: return object[name]
        #    except: pass

        # the object was not found, so output a standard HTML Not Found Error
        REQUEST.RESPONSE.notFoundError(name)


    persistent__getitem__ = ObjectManager.__getitem__
    def __getitem__(self, i):
        '''__getitem__(self, i) --> try to get the item in a dictionary-like syntax; return object'''
        try:
            if (type(i) is StringType):
                return self._getObjectSafely(i)
            else:
                raise TypeError, 'The index must be a string.'
        except:
            if i in self.dbTable.getAllAttributeNames():
                return self.getTemporaryAttributes()[i]
            
        return self.persistent__getitem__(item)


    persistent_setObject = ObjectManager._setObject
    def _setObject(self,id,object,roles=None,user=None, set_owner=1):
        '''_setObject(self,id,object,roles=None,user=None, set_owner=1) --> set the object in the
           container; return None'''
        # no need to set the item, if the object is not saved in anyway
        if self.isVolatile or object.isVolatile:
            pass
        else:
            self.persistent_setObject(id, object, roles, user, set_owner)


    def _getObjectSafely(self, name, default=General._marker):
        '''getObjectSafely(self, name) --> get object from the db; if not throw an informative error;
           return object or HTML'''
        # just to make sure we return a nice HTML error
        try:
            return self._getObject(name, default)
        except (AttributeError, ValueError): 
            HTTPResponse().notFoundError(name)


    def _getOb(self, id, default=General._marker):
        '''_getOb(self, id, default=General._marker) --> get the object; return object'''
        return self._getObject(id, default)


    persistent_getOb = ObjectManager._getOb
    def _getObject(self, name, default=General._marker):
        '''_getObject(self, id, default=_marker) --> get object from the database; return object'''

        # Since the FolderishDBObject should only contain Folders, so guessing the name is easy
        metaType = name[:-1]+'Folder'

        # Check whether the metaType exists; if not, it must have been a regular attribute 
        if metaType in self.allowedSubObjects:
            cl = General.getSubObjectClass(metaType)
            # if the class is volatile, create the object and bind it to the tree
            if cl.isVolatile:
                object =  self.manage_add(subObjectMetaType=metaType, returnObj=1)
                #object._p_jar = self._p_jar
                object = object.__of__(self)
                #object._afterAdd(self)
                return object
            else:
                return self.persistent_getOb(name, default)

        elif name in map(lambda attr: attr.name, self.dbTable.getAllAttributes()):
            if self.getTemporaryAttributes():
                return self.getTemporaryAttributes()[name]
            else:
                return self.getAttributes()[0][name]
            
        else:
            return getattr(self, name)


    persistent_objectIds = ObjectManager.objectIds
    def objectIds(self, spec=None):
        '''objectIds(self, spec=None) --> return the ids of all the sub-objects;
           return list of string'''
        return map(lambda obj: obj.id, self.objectValues(spec))
    

    def objectValues(self, spec=None):
        '''objectValues(self, spec=None) --> return the values of all the sub-objects;
           return list of object'''
        objects = []

        # Since we do not know yet, which objects are volatile and which are not, we
        # have to go through every object meta type and get it.
        for metaType in self.allowedSubObjects:
            cl = General.getSubObjectClass(metaType)
            # if the object is volatile, we need to create one pretty quick
            if cl.isVolatile:
                object = self.manage_add(subObjectMetaType=metaType, returnObj=1)
                # object._p_jar = self._p_jar
                object = object.__of__(self)
                object._afterAdd(self)
                objects.append(object)
            else:
                id = metaType[:-6]+'s'
                if id in self.persistent_objectIds(spec):
                    objects.append(self.persistent_getOb(id))

        return objects
    
            
    def objectItems(self, spec=None):
        '''objectItems(self, spec=None) --> return a list of (id, subobject) tuples;
           return list of tuples'''
        objects = self.objectValues(spec)
        return map(lambda ob: (ob.id, ob), objects)


    def manage_add(self, REQUEST = None, subObjectMetaType = None, returnId = 0, returnObj = 0):
        '''manage_add(self, REQUEST, returnId = 0, subObjectMetaType = None) --> add a sub-object
           to the PlanDepotFolder; return None or new sub-object id'''

        if subObjectMetaType in self.allowedSubObjects:

            self._beforeAdd(self, REQUEST)

            subObjectClass = General.getSubObjectClass(subObjectMetaType)
            subObjectID    = string.replace(subObjectMetaType, 'Folder', 's')
            subObjectTitle = subObjectID + ' Folder'
            subObject      = subObjectClass(self, subObjectID, subObjectTitle)

            self._setObject(subObjectID, subObject)
            if not subObject.isVolatile:
                self[subObjectID]._afterAdd(self, REQUEST)

            if returnId:
                return subObjectID

            if returnObj:
                return subObject
        

        # Try to go back to the folder's default (main) view
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    manage_addForm = manage_add


    def findExistingSubObjects(self):
        '''findExistingSubObjects(self) --> Fill the Project with sub-objects; return None'''

        for metaType in self.allowedSubObjects:
            folder = self.manage_add(subObjectMetaType=metaType, returnObj=1)
            self[folder.id].findExistingSubObjects()


    def manage_delObjects(self, ids=[], REQUEST=None):
        '''manage_delObjects(self, ids) --> delete the objects passed in the id; return None'''
        for id in ids:
            if hasattr(self, id):
	        self._beforeDelete(self, REQUEST)
                try:
                    self[id].deleteDBAttributes()
                except AttributeError:
                    pass
                self._delObject(id)
		self._afterDelete(self, REQUEST)
            else:
                raise "Delete: Object %s not found in object %s" %(id, self.id)        

        if REQUEST:
            return self.manage_main(self, REQUEST)


    deleteOwnDBAttributes = DBObject.deleteDBAttributes


    def deleteDBAttributes(self):
        '''deleteAttributes(self) --> Delete the object in the DB; return None'''

        # first delete all sub-object's DB attributes
        for id in self.objectIds():
            self[id].deleteDBAttributes()

        return self.deleteOwnDBAttributes()


    def getDBContainerOfSubObjects(self):
        '''getDBContainerOfSubObjects(self) --> get the ZODB object of the DB Container;
           return class instance'''

        return self




