# This code is copyrighted by bluedynamics.com and iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: February 4, 2001

import Globals, General
from Products.ZSQLMethods.SQL import SQL
from DBObjectFolder           import DBObjectFolder


class QueryObjectFolderSupport:
    '''QueryObjectFolderSupport class - Abstract description of the DBObject container that uses a specific query'''    


    oldInitSQLMethods = DBObjectFolder.initSQLMethods
    def initSQLMethods(self, container, connectionId='dbConnection'):
        '''initSQLMethods(self, container) --> initializes additional SQML Methods; return None'''

        for sqlMethodDef in [('qry_getOne%sQuery', 'id'), ('qry_getAll%ssQuery', 'table containertype containerid orderBy')]:
            fullSQLMethodId = sqlMethodDef[0] %(self.pseudoSubObjectMetaType)
            if not hasattr(container, fullSQLMethodId):
                setattr(container, fullSQLMethodId, SQL(fullSQLMethodId, '', connectionId,
                                                        sqlMethodDef[1], eval('self._'+fullSQLMethodId)))

        return self.oldInitSQLMethods(container, connectionId)


    def getAllSubObjects(self, metaType=None, orderBy=''):
        '''getAllSubObjects(self) --> retrieve all the sub-objects from the DB; return db rows'''
	
        if not metaType:
            metaType = self.allowedSubObjects[0]

        dbContainer = self._getSQLMethodContainer(self)
        containerId, containerType = self.getDBContainerInfo()

        try:
            return getattr(dbContainer,
                           'qry_getAll%ssQuery' %self.pseudoSubObjectMetaType)(table=metaType,
                                                                              containerid=containerId,
                                                                              containertype=containerType,
                                                                              orderBy=orderBy)
        except:
            return []


    def _getObject(self, name, default=General._marker):
        '''_getObject(self, id, default=General._marker) --> get object from the database; return object'''        
        object = None


        if not (self.isVolatile and General.canStringBeInt(name)):
            # first try to get the object from the ZODB, in case the object is persistent
            object = self.persistent_getOb(name, default)
            if not object: object = None
        
        if (object is None):
            # okay, this failed, so let's see, if it exists in the DB
            dbContainer = self._getSQLMethodContainer(self)

            # since we have no way of determining the meta_type, we need to check all of them
            # Note: Ususally this is only one, so the performance will be fine!
            for metaType in self.allowedSubObjects:
                objectClass = General.getSubObjectClass(metaType)
                try:
                    dbRows = getattr(dbContainer,
                                     'qry_getOne%sQuery' %self.pseudoSubObjectMetaType)(id=name)
                except:
                    dbRows = None

                # If a result was returned, then we got something; now we need to check, whether
                # the object volatile. If so, return it!
                if dbRows and objectClass.isVolatile:
                    object = objectClass(self, dbRow=dbRows[0])
                    object._p_jar = self._p_jar
                    object = object.__of__(self)
                    object._afterAdd(self)

        if (object is None):
            if default is General._marker:
                raise AttributeError, id
            return default

        return object
    

class QueryObjectFolder(QueryObjectFolderSupport, DBObjectFolder):
    '''QueryObjectFolder class - Abstract description of the DBObject container that uses a specific query'''    
    pass


Globals.default__class_init__(QueryObjectFolder)


