# This code is copyrighted by iuveno AG in January, 2001.
#
# written by Stephan Richter
# Last updated: February 22, 2001

import string, base64, urllib

from AccessControl.User    import BasicUserFolder, _noroles, emergency_user, nobody
from DateTime              import DateTime
from UserSupport           import UserSupport


_qry_getPersonById = '''
SELECT *
  FROM Person
  WHERE id = <dtml-sqlvar personid type="int">
'''

_qry_getAllPersonsOfAllGroups = '''
SELECT *
  FROM Person
'''

_qry_getUserWithUserName = '''
SELECT *
  FROM Person
  WHERE %(username)s = <dtml-sqlvar %(username)s type="string">
''' %(UserSupport.authAttributes)


class UserFolderSupport(BasicUserFolder):
    '''UserFolderSupport class - implements a UserFolder for DBObjects'''

    sqlMethods = ( ('qry_getPersonById',            'personid', _qry_getPersonById), 
                   ('qry_getAllPersonsOfAllGroups', '', _qry_getAllPersonsOfAllGroups),
                   ('qry_getUserWithUserName',      UserSupport.authAttributes['username'],
                    _qry_getUserWithUserName))

    authAttributes = UserSupport.authAttributes

    ###  Zope-specific UserFolder API methods  ###

    def getUserNames(self):
        '''getUserNames(self) --> Returns a list of user names; return list of string'''
        container = self._getSQLMethodContainer(self)
        dbRows = container.qry_getAllPersonsOfAllGroups()
        return map(lambda x: x[self.authAttributes['username']], dbRows)

        
    def getUsers(self):
        '''getUsers(self) --> Return a list of person objects; return list of Person'''
        users = []
        for group in self.objectValues():
            users = users + group.Persons.objectValues()

        return users
    

    def getUser(self, name):
        '''getUser(self, name) --> Return the named user object or None if no such user exists;
           return user'''

        if name == self._emergency_user.getUserName():
            return self._emergency_user

        container = self._getSQLMethodContainer(self)
        dbRows = apply(container.qry_getUserWithUserName, (),
                       {self.authAttributes['username']: name})

        if len(dbRows) != 0:
            return self.getObjectTreeWithSQL(self, [dbRows[0].id, 'Persons'], dbRows[0].containertype,
                                             dbRows[0].containerid) 
        else:
            return None


    def validate(self, request, auth='', roles=_noroles, relogin=0):
        '''validate(self, request, auth='', roles=None) --> validate username and password; return Person'''
        if auth is not None and auth[:5] == 'Basic':
            return self.aq_parent.aq_parent.acl_users.validate(request, auth, roles)
        else:
            return self.cookieValidate(request, auth, roles, relogin)


    ###  End of API methods  ###


    def cookieValidate(self, request, auth='', roles=_noroles, relogin=0):
        '''cookie_validate(self, request, auth='', roles=None) --> validates username and password based on
           a cookie; return Person'''

        # setup some local variables, so they can be faster accessed
	parent = request['PARENTS'][0]
        resp   = request['RESPONSE']

        # if the cookie __ac already exists (a user is logged in), then try to decode
        # the __ac string and use the information in there to authenticate the user  
        if request.has_key('__ac') and not relogin:
            c = request['__ac']
            c = urllib.unquote(c)
            try:
                c = base64.decodestring(c)
            except:
                resp.expireCookie('__ac', path='/')
                raise 'LoginIncorrect', 'The authentication string (base64) was incorrect.' 
            
            name, password = tuple(string.split(c, ':'))
            
            # Try to find the user based on the username string in the __ac
            # string
            user = self.getUser(name)
            if user is not None:
                if user.authenticate(password, request):
                    user=user.__of__(self)
                    return user

            resp.expireCookie('__ac', path='/')
            raise 'LoginIncorrect', 'The Username specified in __ac was not found!' 

        # If the REQUEST contains __ac_name and __ac_password try to authenticate
        # using this information; this is used during the first login process for
        # a specific session and will set the __ac cookie, if the user was
        # successfully authenticated
        if request.has_key('__ac_name') and request.has_key('__ac_password'):
            name = request['__ac_name']
            password = request['__ac_password']

            # Find user
            user = self.getUser(name)
            if user is not None:
                if user.authenticate(password, request):
                    user = user.__of__(self)

                    token = '%s:%s' % (name, password)
                    token = base64.encodestring(token)
                    token = urllib.quote(token)
                    resp.setCookie('__ac', token, path='/')
                    request['__ac'] = token
                    try:
                        del request['__ac_name']
                        del request['__ac_password']
                    except:
                        pass
                    return user

            raise 'LoginIncorrect', self.loginForm(self, request) 
        

        if auth:
            return None

        # Try domain matching
        for ob in self.getUsers():
            domains= ob.getDomains()
            if ob.domains:
                if ob.authenticate('', request):
                    if ob.allowed(parent, roles):
                        ob=ob.__of__(self)
                        return ob

        # Allow anonymous access to things under PersonDbs,
        # if the object is public.
        if nobody.allowed(parent, roles):
            ob=nobody.__of__(self)
            return ob

        raise 'LoginRequired', self.loginForm(self, request)


    def logout(self, REQUEST):
        '''logout(self, REQUEST) --> logs out a user; return None'''
        REQUEST['RESPONSE'].expireCookie('__ac', path='/')


    def __len__(self):
        return len(self.objectIds())


    def getAllPersonsOfAllGroups(self):
        '''getAllPersonsOfAllGroups(self) --> Get every person in the DB; return dbRows'''
        container = self._getSQLMethodContainer(self)
        return container.qry_getAllPersonsOfAllGroups()
        
